import { splitToken } from '@shikijs/core';
import Color from 'colorjs.io';

function defaultGetForegroundColor(color) {
  let c;
  try {
    c = new Color(color);
  } catch {
    return null;
  }
  if (c.alpha <= 0.3) {
    return "inherit";
  }
  const [, s, l] = c.hsl;
  if (l > 60) {
    return "#000000";
  } else if (l < 30) {
    return "#ffffff";
  }
  if (s < 40) {
    return l > 40 ? "#000000" : "#ffffff";
  }
  return "#ffffff";
}
function transformerColorHighlight(options = {}) {
  const {
    getForegroundColor = defaultGetForegroundColor,
    htmlStyle = "display:inline-block;padding:0 0.15em;margin:0 -0.15em;border-radius:0.2em;"
  } = options;
  const map = /* @__PURE__ */ new WeakMap();
  return {
    name: "shiki-transformer-color-highlight",
    preprocess(code) {
      const usages = detectColorUsage(code, this.options.lang).map((i) => ({
        ...i,
        foreground: getForegroundColor(i.color)
      })).filter((i) => i.foreground);
      map.set(this.meta, usages);
      return void 0;
    },
    tokens(tokens) {
      const usages = map.get(this.meta);
      if (!usages?.length)
        return;
      return tokens.map((line) => {
        if (!line.length)
          return line;
        const start = line[0];
        const end = line[line.length - 1];
        const lineUsages = usages.filter((i) => start.offset < i.start && i.end < end.offset + end.content.length).sort((a, b) => a.start - b.start);
        const breakpoints = lineUsages.flatMap((i) => [i.start, i.end]);
        if (!lineUsages.length) {
          return line;
        }
        const splitted = line.flatMap((token) => {
          const localBreakpoints = breakpoints.filter((i) => token.offset <= i && i < token.offset + token.content.length).map((i) => i - token.offset);
          return splitToken(token, localBreakpoints);
        });
        const tokens2 = [];
        const buffer = [];
        let current;
        function handleBuffer() {
          if (current) {
            if (!buffer.length) {
              throw new Error("Unexpected empty buffer");
            }
            const token = {
              offset: buffer[0].offset,
              content: buffer.map((i) => i.content).join(""),
              bgColor: current.color,
              color: current.foreground,
              htmlStyle: `background-color:${current.color};color:${current.foreground};${htmlStyle}`
            };
            tokens2.push(token);
            buffer.length = 0;
            current = void 0;
          }
        }
        for (const t of splitted) {
          const usage = lineUsages.find((i) => i.start <= t.offset && t.offset + t.content.length <= i.end);
          if (!usage) {
            handleBuffer();
            tokens2.push(t);
            continue;
          }
          current = usage;
          buffer.push(t);
        }
        handleBuffer();
        return tokens2;
      });
    }
  };
}
const HEXRegex = /#[0-9a-f]{3,8}\b/gi;
const RGBHSLRegex = /\b(?:rgb|hsl)a?\([\d\s\-,./%]+\)/gi;
function detectColorUsage(code, _lang) {
  const usages = [];
  for (const match of code.matchAll(HEXRegex)) {
    const color = match[0];
    if (![3, 4, 6, 8].includes(color.length - 1)) {
      continue;
    }
    const start = match.index;
    const end = start + color.length;
    usages.push({ start, end, color });
  }
  for (const match of code.matchAll(RGBHSLRegex)) {
    const color = match[0];
    const start = match.index;
    const end = start + color.length;
    usages.push({ start, end, color });
  }
  return usages.sort((a, b) => a.start - b.start);
}

export { defaultGetForegroundColor, detectColorUsage, transformerColorHighlight };
